; RM
; 512b SID music
; by Stefano Tognon 2006
; This is a cover of David W. "Red Max"

; For having all the high/low frequency of notes, I use the inverse
; thecnique of Ivan Del Duca (he starts from octave 8 and go down, 
; I start from octave 0 and goes up, saving 12 bytes for high values
; as they are to take all = 1)
; The base routine take about 41 bytes, some overhead is due to the use 
; of transpositions of note

; note sequence are choosed for having a transpose simple by adding the value
; to octave/note. Fortunately only 2 bytes are needed for this

; devel note: AD SR could be combined as low/high nibble are in right position

 .processor 6502 

actFreqLow  = $10
actFreqHigh = $11


curTrack = $57   ; this and +7 and +14 are 0 at sturtup of after loading current track/transposition position
curPat   = $58   ; this and +7 and +14 are 0 at sturtup of after loading 

temp2    = $2A   
delay    = $2B   ; init at 1
temp     = $2C
duration = $2D   ; actual duration +0 +7 +14
storeDur = $2E   ; stored duration to use +0 +7 +14 
instr    = $2F   ; instrument number to use +0 +7 +14 
transp   = $30   ; transposition for this pattern +0 +7 +14
DUR = 48

  .org $0326  

  .byte $2A, $03
  .byte $ED, $F6 

  .org $032A

      sei                    ; disable interrupt


  lda #$8
  ;sta  $d402
  sta  $d403
  ;sta  $d402+7
  sta  $d403+7
  ;sta  $d402+14
  sta  $d403+14
      lda  trPos               ; read initial track position
      sta  curTrack
      lda  trPos+1             ; read initial track position
      sta  curTrack+7
      lda  trPos+3             ; read initial track position
      sta  curTrack+14
  lda #1
  sta  duration
  sta  duration+7
  sta duration+14

sync:
     ;lda  #$ff
loopSync:
     cpx  $D012
     bne  loopSync

      ldx  #DUR
      dec  delay
      bne  sync
      stx  delay 

      ldx  #14                ; start from last voice
loopVoice:
      dec  duration,x
      bne  nextVoice
      lda  storeDur,x
      sta  duration,x

nextNote:
      lda #$40
      sta $D404,x
                             ; read next command
      ldy  curPat,x      
      lda  pat,y
      beq  nextTrack

      bpl  putNote
      cmp  #$C0               ; len/instr?
      bcc  newInst

      and  #$3F               ; store duration
      sta  storeDur,x
      sta  duration,x

      inc  curPat,x
      bne  nextNote

newInst:
      and  #$0F
      sta  instr,x            ; store instrument to use
      inc  curPat,x
      bpl  nextNote
 

putNote:
      jsr  noteFreq
      lda  actFreqLow
      sta  $D400,x
      lda  actFreqHigh
      sta  $D401,x
      ldy  instr,x             ; instrument number
      lda  control,y
      sta  $D404,x            ; put control sid value
      lda  ad,y
      sta  $D405,x            ; put attack/decay
      lda  sr,y
      sta  $D406,x            ; put sustain/release
      inc  curPat,x


; increment voice index and test for last voice      
nextVoice:

      lda  #$7f
      sta  $D418

      .byte $cb,7            ; sbx #7 
      bpl  loopVoice
      bmi  sync


nextTrack:
      ldy  curTrack,x
      lda  trans,y
      sta  transp,x              ; store transposition for this pattern
      bmi  resetTrack

      cpx  #7
      beq  tr2
      bpl  tr3
      lda  #(pat0-pat)           ; fix pattern for voice 1

setPat:
      sta  curPat,x              ; store current pattern
      inc  curTrack,x
      bpl  nextNote
resetTrack:
      txa
      lsr
      lsr
      tay
      lda  trPos,y               ; read initial track position
      sta  curTrack,x
      ;sta  curPat,x
      bpl  nextNote

tr2:                             ; fix pattern for voice 2
      lda  #(pat1-pat)
      jmp  setPat

tr3:                             ; voice 3 use different patterns
      lda  track,y
      jmp  setPat  

  
noteFreq:
     stx  temp
; set frequency A=octave/note
     clc
     adc  transp,x  
     pha

     and  #$0F
     tax
     lda  freqSource,x
     sta  actFreqLow
     lda  #1
     sta  actFreqHigh

     pla
     and  #$F0                   ; take the octave of note
     lsr
     ;.byte $6B, $F0             ; ARR #$0F like the above as C=0
     ;.byte $4b, $F0              ; ASR #$0F
     lsr
     lsr
     lsr
     tay
     beq  freqOk
calcFreq:
     asl  actFreqLow
     rol  actFreqHigh
     dey
     bne  calcFreq
freqOk:
     ldx  temp
     rts

fC0  = 12
fC_0 = 28
fD0  = 45
fD_0 = 62
fE0  = 81
fF0  = 102
fF_0 = 123
fG0  = 145
fG_0 = 169
fA0  = 195
fA_0 = 221
fB0  = 250


; source values used to calculate frequency table.   (taken from 4mat)
freqSource  .byte fC0, fD0, fE0, fD_0, fF0, fG0, fA0, fG_0, fA_0, fC_0, fF_0, fB0,   fG0, fA0 


; instrument
control:
  .byte $41, $11, $81, $41, $41, $41, $41, $41, $41
ad:
  .byte $0A, $09, $09, $08, $00, $08, $09, $0a, $0C
sr:
  .byte $40, $00, $00, $00, $30, $40, $40, $60, $80

; voice 1:
I1 = 0  ; 41 0a40
I2 = 1  ; 11 0900
I3 = 2  ; 81 0900

; voice 2
I4 = 3  ; 41 0800

; voice 3
I5 = 4  ; 41 0030
I6 = 5  ; 41 0840
I7 = 6  ; 41 0940
I8 = 7  ; 41 0a60
I9 = 8  ; 41 0c80

; length:
; 1 2 3 4 6 7 16 17
L1  = 1
L2  = 2
L3  = 3
L4  = 4
L6  = 6
L7  = 7
L16 = 16
L17 = 17
L32 = 32

INS = $80        ; instrument
LEN = $C0        ; length

C0  = 0
D0  = 1
E0  = 2
D_0 = 3
F0  = 4
G0  = 5
A0  = 6
G_0 = 7
A_0 = 8
C_0 = 9
F_0 = 10
B0  = 11
 

; notes:
C1  = $10 + C0  ; 12
F1  = $10 + F0  ; 17
A_1 = $10 + A_0 ; 22
C2  = $20 + C0  ; 24
A_2 = $20 + A_0 ; 34
C3  = $30 + C0  ; 36
D3  = $30 + D0  ; 38
E3  = $30 + E0  ; 40
F3  = $30 + F0  ; 41
G3  = $30 + G0  ; 43
G_3 = $30 + G_0 ; 44
A3  = $30 + A0  ; 45
A_3 = $30 + A_0 ; 46
C4  = $40 + C0  ; 48 
D4  = $40 + D0  ; 50
E4  = $40 + E0  ; 52
G4  = $40 + G0  ; 55 
C5  = $50 + C0  ; 60
G5  = $50 + G0  ; 67 
G_5 = $50 + G_0 ; 68
C6  = $60 + C0  ; 72
D6  = $60 + D0  ; 74
F6  = $60 + F0  ; 77

pat:
  .byte $00  ; this must be zero as all pattern start from here (already init at 0)

pat0:
  .byte INS+I1, LEN+L2, F1
  .byte INS+I2, LEN+L1, G_5, C6
  .byte INS+I3, LEN+L2, C6
  .byte INS+I1,         C2, F1
  .byte INS+I2, LEN+L1, G_5, G5
  .byte INS+I3, LEN+L2, C5
  .byte INS+I1,         C1, 0

pat1:
  .byte INS+I4, LEN+L1
  .byte A_1, A_1, A_1, A_2, A_1, A_1, A_1, A_1
  .byte A_1, A_1, A_1, A_2, A_1, A_1, A_2, A_2, 0

pat2:
  .byte INS+I5, LEN+L16, F6, F6, C6, C6, F6, F6, F6, C6, 0 

pat3:
  .byte INS+I6, LEN+L1, E3, F3
  .byte INS+I7, LEN+L2, F3, F3, F3
  .byte         LEN+L6, F3
  .byte         LEN+L2, C3
  .byte INS+I6, LEN+L1, F3, C4
  .byte INS+I7, LEN+L2, C4
  .byte         LEN+L3, C4
  .byte         LEN+L1, C3
  .byte INS+I6,         C4, C4
  .byte INS+I7, LEN+L2, C4, A_3
  .byte         LEN+L1, G_3
  .byte INS+I8, LEN+L17,G3
  .byte INS+I6, LEN+L1, G3, E3, C3, C4, G3, E3, E4, C4, G3, G4, E4, C4, C5, G4, E4, C4, 0

pat4:
  .byte INS+I9, LEN+L16, D3, A_2, 0

pat5:
  .byte INS+I6, LEN+L1, A3, A3
  .byte INS+I7, LEN+L2, G3, A3
  .byte         LEN+L1, G3
  .byte         LEN+L2, A3
  .byte         LEN+L3, D4
  .byte         LEN+L4, D6
  .byte INS+I6, LEN+L1, F3, F3
  .byte INS+I7, LEN+L2, E3, F3
  .byte         LEN+L1, E3
  .byte         LEN+L3, F3
  .byte         LEN+L2, A3, F3, E3, 0

pat6:
  .byte INS+I9, LEN+L32, E3, 0



; track1 transpose
trans:
; track3 transpose
trans3:
  .byte 0, $10, 0, 0, 0, 0, 0, $FF

; track1 transpose
trans1:
  .byte 0, 0, 12, 12, 0, 0, 12, 12
  .byte 12, 12, 1, 1, 12, 12, 1, 1
  .byte 13, 13, 0, 0, 13, 13, 12, 12
  .byte $FF

; track2 transpose
trans2:
  .byte 12, 12, 8, 8, 12, 12, 8, 8
  .byte 12, 12, 8, 8, 12, 12, 8, 8
  .byte 9, 9, 0, 0, 9, 9, 8, 8
  .byte $FF

trPos:
  .byte (trans1-trans), (trans2-trans), 0, 0

track:
  .byte (pat2-pat), (pat3-pat), (pat3-pat), (pat5-pat), (pat4-pat), (pat5-pat), (pat6-pat)